#!/bin/sh

# default_fw_program.sh - tool to update various devices by using
#                         default firmware images from file system

. /usr/local/bin/sig_catcher

REV_VESPA_PRODUCTION="2"
VESPA_PROD_ONLY_SEQ="Only production hardware can have the seq CPLD updated"

vespa_board_rev() 
{
local val

	val=""
	val=$(cat /var/run/boardrev 2> /dev/null)

	if [ -z "$val" ] ; then
		val=0
	fi

    echo ${val}

}

usage() {
local rev

    rev="$(vespa_board_rev)"
    
	case `cat /var/run/platform` in
	Vespa)
        if [ "${rev}" -ge "$REV_VESPA_PRODUCTION" ] ; then 
    		echo "please supply a device to update: all, switch, nvmem, acp, ebod or seq"
        else
    		echo "please supply a device to update: all, switch, nvmem, acp or ebod"
        fi
		;;
	Helix|Zuma)
    		echo "please supply a device to update: all, switch, acp, ebod, nvmem or seq"
		;;
	*)
		echo "Command not supported on this platform"
		;;
	esac
   	exit 1 
}

if [ $# -ne 1 ]; then
    usage
    exit 1
fi

# only allow updates if the bootloader is running
if [ "$(cat /sys/class/hwmon/bbsp)" != "20" ]; then
	echo "The cplds can only be updated when the system is at the Loader to ensure no loss of customer data"
	exit 1
fi

#
# VESPA specific (also Zuma and Helix)
#
VESPA_CPLD_PROG_TOOL=/usr/local/bin/vespa_cpld_program.sh



validate_image()
# $1: path
# $2: image pattern
# $3: device
#
{
	cd $1
	file_count=`ls $2 | wc -w`

	if [ $file_count != 1 ] ; then
		echo $3 device: unexpected $file_count firmware files found!
		echo `ls $2` 
		echo
		echo Aborting update! 
		exit 1
	fi
}

fw_device_update()
# $1: device
{

local retval

	retval=0

	echo 
	echo "Updating $1:"
	echo 

	case `cat /var/run/platform` in
	Vespa)
                CPLD_PWD=/firmware/Vespa/cpld
		case $1 in
		switch)
			echo switch_program.sh 0
			if [ -z "$SSH_TTY" ] ; then
				switch_program.sh 0
			else 
				echo "no update of the switch firmware, as that would terminate your ssh session"
				echo "if you need to update the switch firmware, please do so from a serial console"
			fi	
			;;
		nvmem)
			validate_image $CPLD_PWD vespa_nvm_rev_*.jam $1
			FILE_CPLD=$CPLD_PWD/vespa_nvm_rev_*.jam

			echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		acp)
			validate_image $CPLD_PWD acp_rev_*.jam $1
			FILE_CPLD=$CPLD_PWD/acp_rev_*.jam

			echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		seq)
            # belt and suspenders!! double check that updating the seq CPLD is OK
            if [  "${rev}" -ge "$REV_VESPA_PRODUCTION" ] ; then 
           		validate_image $CPLD_PWD VESPA_SEQ_CPLD_*.jam $1
                FILE_CPLD=$CPLD_PWD/VESPA_SEQ_CPLD_*.jam

                echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			    $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
				retval=$?
            else
                echo "Board rev is ${rev}"
                echo "${VESPA_PROD_ONLY_SEQ}"
            fi
			;;
		ebod)
		    validate_image $CPLD_PWD ebod_rev_*.jam $1
		    FILE_CPLD=$CPLD_PWD/ebod_rev_*.jam

		    echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
		    $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		*)
			echo Unknown device: $1
   			usage
			exit 1
			;;
		esac
		;;
	Zuma)
                CPLD_PWD=/firmware/ATT/cpld
		case $1 in
		switch)
			echo switch_program.sh 0
			if [ -z "$SSH_TTY" ] ; then
				switch_program.sh 0
			else 
				echo "no update of the switch firmware, as that would terminate your ssh session"
				echo "if you need to update the switch firmware, please do so from a serial console"
			fi	
			;;
		nvmem)
			validate_image $CPLD_PWD ATT_NVMEM_*.jam $1
			FILE_CPLD=$CPLD_PWD/ATT_NVMEM_*.jam

			echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		acp)
			validate_image $CPLD_PWD ATT_ACP_SP_*.jam $1
			FILE_CPLD=$CPLD_PWD/ATT_ACP_SP_*.jam

			echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		seq)
			# Different versions of seq cpld code must be loaded for different
			# hardware board revisions.  Need to distinguish between P1 boards
			# (board rev < 0x21), P2 boards (board rev < 0xA0), and pilot 
			# boards (board rev >= 0xA0).
			ver=0x$(/usr/local/bin/fruprog show 0 | grep "Product Version.*$" | cut -c 21-)
                     hexver=$( ( hexver=$((${ver})) &> /dev/null ) && echo $((${ver})) )

			if [ -z $hexver ] ; then
			       echo Invalid product version $ver
		       	retval=1
		       else
                     	if [ $(( $hexver )) -lt $(( 0x20 )) ] ; then
                         		validate_image $CPLD_PWD P1/Zuma_ATT_SEQ_ver_9_5.jam $1
                         		FILE_CPLD=$CPLD_PWD/P1/Zuma_ATT_SEQ_ver_9_5.jam
                     	elif [ $(( $hexver )) -lt $(( 0xA0 )) ] ; then
                         		validate_image $CPLD_PWD P2/Zuma_SEQ_P2_v9_13.jam $1
                         		FILE_CPLD=$CPLD_PWD/P2/Zuma_SEQ_P2_v9_13.jam
                    		else 
                         		validate_image $CPLD_PWD A0/Zuma_ATT_SEQ_v2_3.jam $1
                         		FILE_CPLD=$CPLD_PWD/A0/Zuma_ATT_SEQ_v2_3.jam
                    		fi

                     	echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
                    		$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
                    		retval=$?
                    	fi
                    ;;
		ebod)
		        validate_image $CPLD_PWD ATT_EBOD_*.jam $1
		        FILE_CPLD=$CPLD_PWD/ATT_EBOD_*.jam

		        echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
		       $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		*)
			echo Unknown device: $1
   			usage
			exit 1
			;;
		esac
		;;

	Helix)
                CPLD_PWD=/firmware/ATT/cpld
		case $1 in
		switch)
			echo switch_program.sh 0
			if [ -z "$SSH_TTY" ] ; then
				switch_program.sh 0
			else 
				echo "no update of the switch firmware, as that would terminate your ssh session"
				echo "if you need to update the switch firmware, please do so from a serial console"
			fi	
			;;
		nvmem)
			validate_image $CPLD_PWD ATT_NVMEM_*.jam $1
			FILE_CPLD=$CPLD_PWD/ATT_NVMEM_*.jam

			echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		acp)
			validate_image $CPLD_PWD ATT_ACP_SP_*.jam $1
			FILE_CPLD=$CPLD_PWD/ATT_ACP_SP_*.jam

			echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		seq)
			# Different versions of seq cpld code must be loaded for different
			# hardware board revisions.  Need to distinguish between P1 boards
			# (board rev < 0x21), P2 boards (board rev < 0xA0), and pilot 
			# boards (board rev >= 0xA0).
			ver=0x$(/usr/local/bin/fruprog show 0 | grep "Product Version.*$" | cut -c 21-)
                     hexver=$( ( hexver=$((${ver})) &> /dev/null ) && echo $((${ver})) )

			if [ -z $hexver ] ; then
				echo Invalid product version $ver
		       	retval=1
		       else		       
                     	if [ $(( $hexver )) -lt $(( 0x20 )) ] ; then
                         		validate_image $CPLD_PWD P1/Helix_ATT_SEQ_ver_1_5.jam $1
                         		FILE_CPLD=$CPLD_PWD/P1/Helix_ATT_SEQ_ver_1_5.jam
                     	elif [ $(( $hexver )) -lt $(( 0xA0 )) ] ; then
                         		validate_image $CPLD_PWD P2/Helix_SEQ_P2_v1_10.jam $1
                         		FILE_CPLD=$CPLD_PWD/P2/Helix_SEQ_P2_v1_10.jam
                    		else 
                         		validate_image $CPLD_PWD A0/Helix_ATT_SEQ_v1_12.jam $1
                         		FILE_CPLD=$CPLD_PWD/A0/Helix_ATT_SEQ_v1_12.jam
                    		fi

                     	echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
                     	$VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
                     	retval=$?
                     fi
                     ;;
		ebod)
		        validate_image $CPLD_PWD ATT_EBOD_*.jam $1
		        FILE_CPLD=$CPLD_PWD/ATT_EBOD_*.jam

		        echo $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
		        $VESPA_CPLD_PROG_TOOL $1 $FILE_CPLD
			retval=$?
			;;
		*)
			echo Unknown device: $1
   			usage
			exit 1
			;;
		esac
		;;
	*)
		echo "Command not supported on this platform"
		exit 1
		;;
	esac

	return $retval 

}

#
# main starts here
#
retval=0
case `cat /var/run/platform` in
Vespa)
    rev="$(vespa_board_rev)"
    if [ "$1" != "all" ]; then
	#
	# Specific device update only
	#
        if [  "${rev}" -lt "$REV_VESPA_PRODUCTION" ]  && [ "$1" == "seq" ] ; then
            echo "Board rev is ${rev}"
            echo "${VESPA_PROD_ONLY_SEQ}"
            exit 1 
        else
            fw_device_update $1
			retval=$?
        fi
    else
	#
	# Update all devices
	#
        if [  "${rev}" -ge "$REV_VESPA_PRODUCTION" ] ; then
            components="nvmem acp seq ebod switch"
        else
            echo "${VESPA_PROD_ONLY_SEQ}"
            components="nvmem acp ebod switch"
        fi

		for V_DEV in ${components}
		do
			fw_device_update $V_DEV
			last_retval=$?
			 retval=$(( $retval + $last_retval ))
		done
	fi
	;;
Helix|Zuma)
    rev="$(vespa_board_rev)"
    if [ "$1" != "all" ]; then
	#
	# Specific device update only
	#
        fw_device_update $1
	retval=$?
    else
	#
	# Update all devices
	#
        components="acp seq ebod nvmem switch"

	for V_DEV in ${components}
	        do
		        fw_device_update $V_DEV
		        last_retval=$?
		        retval=$(( $retval + $last_retval ))
	        done
	fi
	;;

*)
	echo "Command not supported on this platform"
	exit 1
	;;
esac
echo
if [ "$1" == "switch" ]; then
	#
	# switch firmware updates do not require power cycle
	#
	echo "Update request completed"
else
	#
	# Any CPLD firmware updates require system power cycle
	#
	if [ "${retval}" == 0 ] ; then 
		echo "Please turn off the power supplies to complete the update process"
	else
		echo "The update process has failed, please contact support."
	fi
fi
exit 0

